--- common/objects.c	Tue Nov 30 18:16:23 2004
+++ common/objects.c	Sun Dec 19 14:57:26 2004
@@ -29,6 +29,9 @@
 
 #ifdef NSCORE
 #include "../include/nagios.h"
+/* get snmp data in main config file to put in hosts */
+extern snmp_data    *snmp_config[MAX_SNMP_DATA];
+extern int          snmp_config_num;
 #endif
 
 #ifdef NSCGI
@@ -923,12 +926,16 @@
 
 
 /* add a new host definition */
-host *add_host(char *name, char *alias, char *address, char *check_period, int check_interval, int max_attempts, int notify_up, int notify_down, int notify_unreachable, int notify_flapping, int notification_interval, char *notification_period, int notifications_enabled, char *check_command, int checks_enabled, int accept_passive_checks, char *event_handler, int event_handler_enabled, int flap_detection_enabled, double low_flap_threshold, double high_flap_threshold, int stalk_up, int stalk_down, int stalk_unreachable, int process_perfdata, int failure_prediction_enabled, char *failure_prediction_options, int check_freshness, int freshness_threshold, int retain_status_information, int retain_nonstatus_information, int obsess_over_host){
+host *add_host(char *name, char *alias, char *address, char *check_period, int check_interval, int max_attempts, int notify_up, int notify_down, int notify_unreachable, int notify_flapping, int notification_interval, char *notification_period, int notifications_enabled, char *check_command, int checks_enabled, int accept_passive_checks, char *event_handler, int event_handler_enabled, int flap_detection_enabled, double low_flap_threshold, double high_flap_threshold, int stalk_up, int stalk_down, int stalk_unreachable, int process_perfdata, int failure_prediction_enabled, char *failure_prediction_options, int check_freshness, int freshness_threshold, int retain_status_information, int retain_nonstatus_information, int obsess_over_host, int snmp_version, char* snmp_login, char* snmp_pass,char** user_defined ){
 	host *temp_host;
 	host *new_host;
+	int i;
 #ifdef NSCORE
 	char temp_buffer[MAX_INPUT_BUFFER];
 	int x;
+	unsigned long mask_hexa,address_hexa;
+	int min_mask,index_mask;
+	char* temp_address;
 #endif
 
 #ifdef DEBUG0
@@ -1308,7 +1315,183 @@
 	else
 		new_host->failure_prediction_options=NULL;
 
+	/* SNMP information */
+        new_host->snmp_version=snmp_version;
+
+	if (snmp_login != NULL && strcmp(snmp_login,"")){
+		new_host->snmp_login=strdup(snmp_login);
+		if(new_host->snmp_login==NULL){
+#ifdef NSCORE
+			snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Could not allocate memory for host '%s' snmp login options\n",name);
+			temp_buffer[sizeof(temp_buffer)-1]='\x0';
+			write_to_logs_and_console(temp_buffer,NSLOG_CONFIG_ERROR,TRUE);
+#endif
+			if(new_host->event_handler!=NULL)
+				free(new_host->event_handler);
+			if(new_host->host_check_command!=NULL)
+				free(new_host->host_check_command);
+			if(new_host->notification_period!=NULL)
+				free(new_host->notification_period);
+			if(new_host->failure_prediction_options!=NULL)
+				free(new_host->failure_prediction_options);
+			free(new_host->check_period);
+			free(new_host->address);
+			free(new_host->alias);
+			free(new_host->name);
+			free(new_host);
+			return NULL;
+	                }
+	        }
+	else
+		new_host->snmp_login=NULL;			
+
+	if (snmp_pass != NULL && strcmp(snmp_pass,"")){
+		new_host->snmp_pass=strdup(snmp_pass);
+		if(new_host->snmp_pass==NULL){
+#ifdef NSCORE
+			snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Could not allocate memory for host '%s' snmp login options\n",name);
+			temp_buffer[sizeof(temp_buffer)-1]='\x0';
+			write_to_logs_and_console(temp_buffer,NSLOG_CONFIG_ERROR,TRUE);
+#endif
+			if(new_host->event_handler!=NULL)
+				free(new_host->event_handler);
+			if(new_host->host_check_command!=NULL)
+				free(new_host->host_check_command);
+			if(new_host->notification_period!=NULL)
+				free(new_host->notification_period);
+			if(new_host->failure_prediction_options!=NULL)
+				free(new_host->failure_prediction_options);
+			if(new_host->snmp_login!=NULL)
+				free(new_host->snmp_login);
+			free(new_host->check_period);
+			free(new_host->address);
+			free(new_host->alias);
+			free(new_host->name);
+			free(new_host);
+			return NULL;
+	                }
+	        }
+	else
+		new_host->snmp_pass=NULL;			
+
+#ifdef NSCORE
+ 	/* if some snmp information is not present, try to check the in the main config file */
+	/* never overwrite existing info but only complete it */
+	if (((snmp_version ==0) || (new_host->snmp_pass==NULL) || (new_host->snmp_login==NULL)) 
+		&& (snmp_config_num > 0 )) {
+		min_mask=0;
+		index_mask=-1;
+		temp_address=strdup(new_host->address);
+		address_hexa= atoi(strtok(temp_address,".")) << 24;
+		address_hexa+= atoi(strtok(NULL,".")) <<16;
+        	address_hexa+= atoi(strtok(NULL,".")) <<8;
+        	address_hexa+= atoi(strtok(NULL,"."));
+		free(temp_address);
+		for (x=0;x<snmp_config_num;x++) {
+			/* compute real mask in hexa : (FFFF FFFF >> hosts_bits) << hosts_bits */	
+			mask_hexa= ( 4294967295UL >> (32- snmp_config[x]->mask)) << (32-snmp_config[x]->mask);
+			if ((mask_hexa & address_hexa) == (snmp_config[x]->address & mask_hexa)) {
+				/* test if the mask if more specific than the last one */
+				/* ex : prefer /32 than /24 mask */
+				if (min_mask<snmp_config[x]->mask) {
+					min_mask=snmp_config[x]->mask;
+					index_mask=x;		
+				}
+			}
 	
+		}
+		if (index_mask >=0) { /* write info only if not present */
+			if (snmp_version ==0) 
+				new_host->snmp_version=snmp_config[index_mask]->version;
+			if (new_host->snmp_pass==NULL) {
+				new_host->snmp_pass=strdup(snmp_config[index_mask]->pass);
+	                	if(new_host->snmp_pass==NULL){
+					/* Note : already in NSCORE definition */
+	                	        snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Could not allocate memory for host '%s' snmp pass options\n",name);
+        	                	temp_buffer[sizeof(temp_buffer)-1]='\x0';
+ 	        	               write_to_logs_and_console(temp_buffer,NSLOG_CONFIG_ERROR,TRUE);
+        	        	        if(new_host->event_handler!=NULL)
+                	        	        free(new_host->event_handler);
+	                        	if(new_host->host_check_command!=NULL)
+        	                        	free(new_host->host_check_command);
+	        	                if(new_host->notification_period!=NULL)
+        	        	                free(new_host->notification_period);
+                	        	if(new_host->failure_prediction_options!=NULL)
+                        	       		free(new_host->failure_prediction_options);
+		                        if(new_host->snmp_login!=NULL)
+        		                        free(new_host->snmp_login);
+                		        free(new_host->check_period);
+                        		free(new_host->address);
+	                        	free(new_host->alias);
+	        	                free(new_host->name);
+        	        	        free(new_host);
+                	        	return NULL;
+                        	}
+			}
+                        if (new_host->snmp_login==NULL) {
+                                new_host->snmp_login=strdup(snmp_config[index_mask]->login);
+                                if(new_host->snmp_login==NULL){
+                                        /* Note : already in NSCORE definition */
+                                        snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Could not allocate memory for host '%s' snmp pass options\n",name);
+                                        temp_buffer[sizeof(temp_buffer)-1]='\x0';
+                                       write_to_logs_and_console(temp_buffer,NSLOG_CONFIG_ERROR,TRUE);
+                                        if(new_host->event_handler!=NULL)
+                                                free(new_host->event_handler);
+                                        if(new_host->host_check_command!=NULL)
+                                                free(new_host->host_check_command);
+                                        if(new_host->notification_period!=NULL)
+                                                free(new_host->notification_period);
+                                        if(new_host->failure_prediction_options!=NULL)
+                                                free(new_host->failure_prediction_options);
+                                        if(new_host->snmp_pass!=NULL)
+                                                free(new_host->snmp_pass);
+                                        free(new_host->check_period);
+                                        free(new_host->address);
+                                        free(new_host->alias);
+                                        free(new_host->name);
+                                        free(new_host);
+                                        return NULL;
+                                }       
+                        }       
+		}
+	
+	}	
+#endif
+	for (i=0;i<MAX_MACRO_HOSTUSER;i++) {
+		if (user_defined[i] !=NULL && strcmp(user_defined[i],"")) {
+			new_host->user_defined[i]=strdup(user_defined[i]);
+			if(new_host->user_defined[i]==NULL) {
+#ifdef NSCORE
+				snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Could not allocate memory for host '%s' snmp login options\n",name);
+				temp_buffer[sizeof(temp_buffer)-1]='\x0';
+				write_to_logs_and_console(temp_buffer,NSLOG_CONFIG_ERROR,TRUE);
+#endif
+				if(new_host->event_handler!=NULL)
+					free(new_host->event_handler);
+				if(new_host->host_check_command!=NULL)
+					free(new_host->host_check_command);
+				if(new_host->notification_period!=NULL)
+					free(new_host->notification_period);
+				if(new_host->failure_prediction_options!=NULL)
+					free(new_host->failure_prediction_options);
+				if(new_host->snmp_pass!=NULL)
+					free(new_host->snmp_pass);
+				if(new_host->snmp_login!=NULL)
+					free(new_host->snmp_login);
+				for (i=0;i<MAX_MACRO_HOSTUSER;i++) {
+					if(new_host->user_defined[i]!=NULL) free(new_host->user_defined[i]);
+				}
+				free(new_host->check_period);
+				free(new_host->address);
+				free(new_host->alias);
+				free(new_host->name);
+				free(new_host);
+				return NULL;
+	        }
+		}
+		else
+		new_host->user_defined[i]=NULL;
+	}
 	new_host->parent_hosts=NULL;
 	new_host->max_attempts=max_attempts;
 	new_host->contact_groups=NULL;
@@ -1394,6 +1577,13 @@
 			free(new_host->host_check_command);
 		if(new_host->notification_period!=NULL)
 			free(new_host->notification_period);
+		if(new_host->snmp_pass!=NULL)
+			free(new_host->snmp_pass);
+		if(new_host->snmp_login!=NULL)
+			free(new_host->snmp_login);
+		for (i=0;i<MAX_MACRO_HOSTUSER;i++) {
+			if(new_host->user_defined[i]!=NULL) free(new_host->user_defined[i]);
+		}
 		free(new_host->address);
 		free(new_host->alias);
 		free(new_host->name);
@@ -1418,6 +1608,13 @@
 			free(new_host->host_check_command);
 		if(new_host->notification_period!=NULL)
 			free(new_host->notification_period);
+		if(new_host->snmp_pass!=NULL)
+			free(new_host->snmp_pass);
+		if(new_host->snmp_login!=NULL)
+			free(new_host->snmp_login);
+		for (i=0;i<MAX_MACRO_HOSTUSER;i++) {
+			if(new_host->user_defined[i]!=NULL) free(new_host->user_defined[i]);
+		}
 		free(new_host->address);
 		free(new_host->alias);
 		free(new_host->name);
@@ -1452,6 +1649,13 @@
 			free(new_host->host_check_command);
 		if(new_host->notification_period!=NULL)
 			free(new_host->notification_period);
+		if(new_host->snmp_pass!=NULL)
+			free(new_host->snmp_pass);
+		if(new_host->snmp_login!=NULL)
+			free(new_host->snmp_login);
+		for (i=0;i<MAX_MACRO_HOSTUSER;i++) {
+			if(new_host->user_defined[i]!=NULL) free(new_host->user_defined[i]);
+		}
 		free(new_host->address);
 		free(new_host->alias);
 		free(new_host->name);
@@ -1487,6 +1691,11 @@
 	printf("\tNotify On Recovery:       %s\n",(new_host->notify_on_recovery==1)?"yes":"no");
 	printf("\tNotification Interval:    %d\n",new_host->notification_interval);
 	printf("\tNotification Time Period: %s\n",(new_host->notification_period==NULL)?"N/A":new_host->notification_period);
+	printf("\tSnmp info:		%i,%s,%s\n",new_host->snmp_version,new_host->snmp_login,new_host->snmp_pass);
+	printf("\tHostUser info:		");
+	for(i=0;i<MAX_MACRO_HOSTUSER;i++)
+		if (new_host->user_defined[i] != NULL) printf("%i:%s ; ",i,new_host->user_defined[i]);
+	printf("\n");
 #endif
 #ifdef DEBUG0
 	printf("add_host() end\n");
--- base/config.c	Thu Dec  9 07:48:33 2004
+++ base/config.c	Sun Dec 19 01:43:39 2004
@@ -145,6 +145,8 @@
 extern double   high_host_flap_threshold;
 
 extern int      date_format;
+extern snmp_data	*snmp_config[MAX_SNMP_DATA];
+extern int			snmp_config_num;
 
 extern contact		*contact_list;
 extern contactgroup	*contactgroup_list;
@@ -1401,6 +1403,26 @@
 			printf("\t\tauth_file set to '%s'\n",auth_file);
 #endif
 		        }
+        /***** SNMP login information *******/
+		else if(!strcmp(variable,"snmp_login")){
+			strip(value);
+			if (process_snmp_config(value)) {
+				strcpy(error_message,"Error parsing snmp login information");
+				error=TRUE;
+				break;
+			        }
+
+#ifdef DEBUG1
+			printf("\t\tsnmp_login parsing (%i):ver %i, Auth: %s/%s, IP/mask: %x/%i\n",
+				snmp_config_num-1,
+				snmp_config[snmp_config_num-1]->version,
+				snmp_config[snmp_config_num-1]->login,
+				snmp_config[snmp_config_num-1]->pass,
+				snmp_config[snmp_config_num-1]->address,
+				snmp_config[snmp_config_num-1]->mask				
+				);
+#endif
+		        }
 
 		/* ignore old/external variables */
 		else if(!strcmp(variable,"status_file"))
@@ -2805,3 +2827,71 @@
 	return (errors>0)?ERROR:OK;
 	}
 
+int process_snmp_config(char *value) {
+	/* return value : 0/OK, 1/Errorparsing, 2/error mem */
+	char *temp,*temp2;
+	char *login,*pass;
+	int version,mask;
+	unsigned long int address;
+
+	/* test if maximum snmp data is not reached */
+	if (snmp_config_num >= MAX_SNMP_DATA) return 1;
+	temp=strtok(value,",\n");
+	if (temp == NULL) return 1;
+    version=atoi(temp);
+	if (version ==1)
+	{
+		/* get snmp community (v1) */
+		if ( (temp=strtok(NULL,",\n")) == NULL ) return 1;
+		if (strlen(temp) > 30) return 1;
+		if ( (pass=strdup(temp)) ==NULL) return 2;
+
+	} else if (version == 3)
+	{ 
+		/* get snmp login (v3) */
+		if ( (temp=strtok(NULL,",\n")) == NULL ) return 1;
+		if (strlen(temp) > 30) return 1;
+		if ( (login=strdup(temp)) ==NULL) return 1;
+		/* get snmp pass (v3) */
+		if ( (temp=strtok(NULL,",\n")) == NULL ) return 1;
+		if (strlen(temp) > 30) return 1;
+		if ( (pass=strdup(temp)) ==NULL) return 1;
+		
+	} else return 1;  /* unknown version */      
+
+	/* get address */
+	if ( (temp=strtok(NULL,"/\n")) == NULL ) return 1;
+	if ( (temp2=strtok(NULL,"\n")) == NULL ) return 1;
+
+	/* convert address to decimal */
+	address= atoi(strtok(temp,".")) << 24;
+	address+= atoi(strtok(NULL,".")) <<16;
+	address+= atoi(strtok(NULL,".")) <<8;
+	address+= atoi(strtok(NULL,"."));
+
+	/* very basic check of address */
+	if (address < 16777216) return 1;
+
+	/* get mask and check */
+	mask=atoi(temp2);
+	if ((mask < 0 ) || (mask > 32)) return 1;
+ 
+	//printf("Version : %i, pass : %s, mask : %x/%i\n",version, pass,address,mask);
+	snmp_config[snmp_config_num]=(snmp_data *) malloc(sizeof(snmp_data));
+   
+	if (snmp_config[snmp_config_num] ==NULL) return 2;
+	snmp_config[snmp_config_num]->version=version;
+	if (version==1) 
+		snmp_config[snmp_config_num]->login=NULL;
+	else {
+		snmp_config[snmp_config_num]->login=strdup(login);
+		if (snmp_config[snmp_config_num]->login ==NULL) return 2;
+	}
+	snmp_config[snmp_config_num]->pass=strdup(pass);
+	if (snmp_config[snmp_config_num]->pass ==NULL) return 2;
+	snmp_config[snmp_config_num]->address=address;
+	snmp_config[snmp_config_num]->mask=mask;
+	snmp_config_num++;
+
+	return 0;
+}
--- base/nagios.c	Thu Dec 16 02:15:25 2004
+++ base/nagios.c	Sun Dec 19 12:08:04 2004
@@ -183,6 +183,9 @@
 double          low_host_flap_threshold=DEFAULT_LOW_HOST_FLAP_THRESHOLD;
 double          high_host_flap_threshold=DEFAULT_HIGH_HOST_FLAP_THRESHOLD;
 
+snmp_data	*snmp_config[MAX_SNMP_DATA];
+int		snmp_config_num=0;
+
 int             date_format=DATE_FORMAT_US;
 
 int             command_file_fd;
--- base/utils.c	Thu Dec 16 06:24:10 2004
+++ base/utils.c	Sun Dec 19 15:22:46 2004
@@ -686,6 +686,7 @@
 	int hours;
 	int minutes;
 	int seconds;
+	int x;
 	char temp_buffer[MAX_INPUT_BUFFER];
 
 #ifdef DEBUG0
@@ -944,6 +945,39 @@
 		free(macro_x[MACRO_HOSTNOTESURL]);
 		macro_x[MACRO_HOSTNOTESURL]=strdup(temp_buffer);
 	        }
+	/* snmp information */
+	if(macro_x[MACRO_SNMP_LOGIN]!=NULL)
+		free(macro_x[MACRO_SNMP_LOGIN]);
+	if (hst->snmp_login == NULL)
+		macro_x[MACRO_SNMP_LOGIN]=NULL;
+	else
+		macro_x[MACRO_SNMP_LOGIN]=strdup(hst->snmp_login);
+	
+	if(macro_x[MACRO_SNMP_PASS]!=NULL)
+		free(macro_x[MACRO_SNMP_PASS]);
+	if (hst->snmp_pass == NULL)
+		macro_x[MACRO_SNMP_PASS]=NULL;
+	else
+		macro_x[MACRO_SNMP_PASS]=strdup(hst->snmp_pass);
+
+	if(macro_x[MACRO_SNMP_VERSION]!=NULL)
+		free(macro_x[MACRO_SNMP_VERSION]);
+	if (hst->snmp_version == 0)
+		macro_x[MACRO_SNMP_VERSION]=NULL;
+	else {
+	  macro_x[MACRO_SNMP_VERSION]=(char *) malloc(sizeof(char)*10);
+	  sprintf(macro_x[MACRO_SNMP_VERSION],"%i",hst->snmp_version);
+	}
+	for (x=0;x<MAX_MACRO_HOSTUSER;x++) {
+		if(macro_x[x+MACRO_HOSTUSER_START]!=NULL)
+			free(macro_x[x+MACRO_HOSTUSER_START]);
+		if (hst->user_defined[x] == NULL)
+			macro_x[x+MACRO_HOSTUSER_START]=NULL;
+		else {
+			macro_x[x+MACRO_HOSTUSER_START]=strdup(hst->user_defined[x]);
+		}
+	}
+
 
 #ifdef DEBUG0
 	printf("grab_host_macros() end\n");
@@ -2231,6 +2265,7 @@
 /* initializes the names of macros */
 int init_macrox_names(void){
 	int x;
+	char temp_macro[15];
 
 #ifdef DEBUG0
 	printf("init_macrox_names() start\n");
@@ -2337,7 +2372,14 @@
 	add_macrox_name(MACRO_TOTALSERVICESUNKNOWNUNHANDLED,"TOTALSERVICESUNKNOWNUNHANDLED");
 	add_macrox_name(MACRO_TOTALSERVICEPROBLEMS,"TOTALSERVICEPROBLEMS");
 	add_macrox_name(MACRO_TOTALSERVICEPROBLEMSUNHANDLED,"TOTALSERVICEPROBLEMSUNHANDLED");
-
+	/* snmp & userdefined macros */
+	add_macrox_name(MACRO_SNMP_VERSION,"SNMPVERSION");  
+	add_macrox_name(MACRO_SNMP_LOGIN,"SNMPLOGIN");
+	add_macrox_name(MACRO_SNMP_PASS,"SNMPPASS");
+	for (x=0;x<MAX_MACRO_HOSTUSER;x++) {
+		sprintf(temp_macro,"HOSTUSER%i",x);
+		add_macrox_name(x+MACRO_HOSTUSER_START,temp_macro);
+	}
 #ifdef DEBUG0
 	printf("init_macrox_names() end\n");
 #endif
--- include/common.h	Thu Dec 16 02:15:27 2004
+++ include/common.h	Sun Dec 19 01:44:52 2004
@@ -327,7 +327,8 @@
 #define READ_SERVICEGROUPS              16384
 
 #define READ_ALL_OBJECT_DATA            READ_HOSTS | READ_HOSTGROUPS | READ_CONTACTS | READ_CONTACTGROUPS | READ_SERVICES | READ_COMMANDS | READ_TIMEPERIODS | READ_SERVICEESCALATIONS | READ_SERVICEDEPENDENCIES | READ_HOSTDEPENDENCIES | READ_HOSTESCALATIONS | READ_HOSTEXTINFO | READ_SERVICEEXTINFO | READ_SERVICEGROUPS
-
+#define MAX_MACRO_HOSTUSER			10
+#define MACRO_HOSTUSER_START		200
 
 /************************** DATE/TIME TYPES *****************************/
 
--- include/objects.h	Sat Nov  6 06:44:13 2004
+++ include/objects.h	Sat Dec 18 17:14:37 2004
@@ -147,6 +147,10 @@
 	int     failure_prediction_enabled;
 	char    *failure_prediction_options;
 	int     obsess_over_host;
+	int		snmp_version;
+	char	*snmp_login;
+	char	*snmp_pass;
+	char 	*user_defined[MAX_MACRO_HOSTUSER];
 #ifdef NSCORE
 	int     problem_has_been_acknowledged;
 	int     acknowledgement_type;
@@ -156,7 +160,7 @@
 	int     last_hard_state;
 	char	*plugin_output;
 	char    *perf_data;
-        int     state_type;
+    int     state_type;
 	int     current_attempt;
 	double  latency;
 	double  execution_time;
@@ -516,7 +520,7 @@
 contact *add_contact(char *,char *,char *,char *,char **,char *,char *,int,int,int,int,int,int,int,int,int);	/* adds a contact definition */
 commandsmember *add_service_notification_command_to_contact(contact *,char *);				/* adds a service notification command to a contact definition */
 commandsmember *add_host_notification_command_to_contact(contact *,char *);				/* adds a host notification command to a contact definition */
-host *add_host(char *,char *,char *,char *,int,int,int,int,int,int,int,char *,int,char *,int,int,char *,int,int,double,double,int,int,int,int,int,char *,int,int,int,int,int);	/* adds a host definition */
+host *add_host(char *,char *,char *,char *,int,int,int,int,int,int,int,char *,int,char *,int,int,char *,int,int,double,double,int,int,int,int,int,char *,int,int,int,int,int,int,char*,char*,char**);	/* adds a host definition */
 hostsmember *add_parent_host_to_host(host *,char *);							/* adds a parent host to a host definition */
 contactgroupsmember *add_contactgroup_to_host(host *,char *);					        /* adds a contactgroup to a host definition */
 timeperiod *add_timeperiod(char *,char *);								/* adds a timeperiod definition */
--- include/nagios.h.in	Thu Dec  9 07:48:34 2004
+++ include/nagios.h.in	Sun Dec 19 01:46:07 2004
@@ -36,6 +36,7 @@
 
 #define MAX_COMMAND_ARGUMENTS			32	/* maximum number of $ARGx$ macros */
 #define MAX_USER_MACROS				256	/* maximum number of $USERx$ macros */
+#define MAX_SNMP_DATA				200 /* maximum number of snmp data */
 
 #define MAX_STATE_LENGTH			32	/* length definitions used in macros */
 #define MAX_STATETYPE_LENGTH			24
@@ -52,7 +53,7 @@
 
 #define MACRO_ENV_VAR_PREFIX			"NAGIOS_"
 
-#define MACRO_X_COUNT				96	/* size of macro_x[] array */
+#define MACRO_X_COUNT    MACRO_HOSTUSER_START+MAX_MACRO_HOSTUSER+1      /* size of macro_x[] array */
 
 #define MACRO_HOSTNAME				0
 #define MACRO_HOSTALIAS				1
@@ -150,6 +151,11 @@
 #define MACRO_TOTALSERVICESUNKNOWNUNHANDLED	93
 #define MACRO_TOTALSERVICEPROBLEMS		94
 #define MACRO_TOTALSERVICEPROBLEMSUNHANDLED	95
+#define MACRO_SNMP_VERSION  96
+#define MACRO_SNMP_LOGIN 97
+#define MACRO_SNMP_PASS 98
+
+
 
 
 
@@ -472,6 +478,15 @@
 #define COMMAND_WORKER_THREAD		  0
 #define SERVICE_WORKER_THREAD  		  1
 
+/* snmp data stucture */
+typedef struct snmp_data_struct{
+  int version;
+  char *login;
+  char *pass;
+  unsigned long address;
+  int  mask;
+        } snmp_data;
+
 
 /******************** FUNCTIONS **********************/
 
@@ -479,6 +494,7 @@
 int read_main_config_file(char *);                     	/* reads the main config file (nagios.cfg) */
 int read_resource_file(char *);				/* processes macros in resource file */
 int read_all_object_data(char *);			/* reads all object config data */
+int process_snmp_config(char *);			/* process snmp configuration */
 
 
 /**** Setup Functions ****/
--- xdata/xodtemplate.c	Sun Dec  5 00:54:13 2004
+++ xdata/xodtemplate.c	Sun Dec 19 01:48:18 2004
@@ -793,6 +793,7 @@
 /* starts a new object definition */
 int xodtemplate_begin_object_definition(char *input, int options, int config_file, int start_line){
 	int result=OK;
+	int user_defined_n;
 	xodtemplate_timeperiod *new_timeperiod;
 	xodtemplate_command *new_command;
 	xodtemplate_contactgroup *new_contactgroup;
@@ -1273,6 +1274,11 @@
 		new_host->register_object=TRUE;
 		new_host->_config_file=config_file;
 		new_host->_start_line=start_line;
+		new_host->snmp_version=0;
+		new_host->snmp_login=NULL;
+		new_host->snmp_pass=NULL;
+		for (user_defined_n=0;user_defined_n< MAX_MACRO_HOSTUSER;user_defined_n++)
+			new_host->user_defined[user_defined_n]=NULL;
 
 		/* add new host to head of list in memory */
 		new_host->next=xodtemplate_host_list;
@@ -2765,6 +2771,45 @@
 		        }
 		else if(!strcmp(variable,"register"))
 			temp_host->register_object=(atoi(value)>0)?TRUE:FALSE;
+		/* get snmp information */
+		else if(!strcmp(variable,"snmp_version"))
+			temp_host->snmp_version=atoi(value);		        
+		else if(!strcmp(variable,"snmp_login")){
+			temp_host->snmp_login=strdup(value);
+			if(temp_host->snmp_login==NULL){
+#ifdef DEBUG1
+				printf("Error: Could not allocate memory for host snmp_login.\n");
+#endif
+				return ERROR;
+			        }
+		        }
+		else if(!strcmp(variable,"snmp_pass")){
+			temp_host->snmp_pass=strdup(value);
+			if(temp_host->snmp_pass==NULL){
+#ifdef DEBUG1
+				printf("Error: Could not allocate memory for host snmp_pass.\n");
+#endif
+				return ERROR;
+			        }
+		        }
+		/* get user host information */
+		else if(!strncmp(variable,"hostuser",8)) {
+			temp_ptr=variable+8;
+			if ((atoi (temp_ptr) < 0) || (atoi(temp_ptr) >= MAX_MACRO_HOSTUSER)) {
+#ifdef DEBUG1
+				printf("Error: hostuser out of bounds.\n");
+#endif		
+				return ERROR;
+			}
+			temp_host->user_defined[atoi(temp_ptr)]=strdup(value);
+			if (temp_host->user_defined[atoi(temp_ptr)]==NULL) {
+#ifdef DEBUG1
+				printf("Error: Could not allocate memory for host hostuser.\n");
+#endif
+				return ERROR;
+			}
+
+		}
 		else{
 #ifdef NSCORE
 			snprintf(temp_buffer,sizeof(temp_buffer)-1,"Error: Invalid host object directive '%s'.\n",variable);
@@ -5786,6 +5831,7 @@
 /* resolves a host object */
 int xodtemplate_resolve_host(xodtemplate_host *this_host){
 	xodtemplate_host *template_host;
+	int i;
 #ifdef NSCORE
 	char temp_buffer[MAX_XODTEMPLATE_INPUT_BUFFER];
 #endif
@@ -5922,7 +5968,19 @@
 		this_host->retain_nonstatus_information=template_host->retain_nonstatus_information;
 		this_host->have_retain_nonstatus_information=TRUE;
 	        }
-
+	/* first snmp template : try host template */
+	if(this_host->snmp_version==0 && template_host->snmp_version!=0)
+		this_host->snmp_version=template_host->snmp_version;
+	if(this_host->snmp_login==NULL && template_host->snmp_login!=NULL)
+		this_host->snmp_login=strdup(template_host->snmp_login);
+	if(this_host->snmp_pass==NULL && template_host->snmp_pass!=NULL)
+		this_host->snmp_pass=strdup(template_host->snmp_pass);
+	/* hostuser template definitions */
+	for (i=0;i< MAX_MACRO_HOSTUSER;i++)
+		if (this_host->user_defined[i]==NULL && template_host->user_defined[i]!=NULL)
+			this_host->user_defined[i]=strdup(template_host->user_defined[i]);
+		
+	
 #ifdef DEBUG0
 	printf("xodtemplate_resolve_host() end\n");
 #endif
@@ -7854,7 +7912,8 @@
 		return OK;
 
 	/* add the host definition */
-	new_host=add_host(this_host->host_name,this_host->alias,(this_host->address==NULL)?this_host->host_name:this_host->address,this_host->check_period,this_host->check_interval,this_host->max_check_attempts,this_host->notify_on_recovery,this_host->notify_on_down,this_host->notify_on_unreachable,this_host->notify_on_flapping,this_host->notification_interval,this_host->notification_period,this_host->notifications_enabled,this_host->check_command,this_host->active_checks_enabled,this_host->passive_checks_enabled,this_host->event_handler,this_host->event_handler_enabled,this_host->flap_detection_enabled,this_host->low_flap_threshold,this_host->high_flap_threshold,this_host->stalk_on_up,this_host->stalk_on_down,this_host->stalk_on_unreachable,this_host->process_perf_data,this_host->failure_prediction_enabled,this_host->failure_prediction_options,this_host->check_freshness,this_host->freshness_threshold,this_host->retain_status_information,this_host->retain_nonstatus_information,this_host->obsess_over_host);
+	new_host=add_host(this_host->host_name,this_host->alias,(this_host->address==NULL)?this_host->host_name:this_host->address,this_host->check_period,this_host->check_interval,this_host->max_check_attempts,this_host->notify_on_recovery,this_host->notify_on_down,this_host->notify_on_unreachable,this_host->notify_on_flapping,this_host->notification_interval,this_host->notification_period,this_host->notifications_enabled,this_host->check_command,this_host->active_checks_enabled,this_host->passive_checks_enabled,this_host->event_handler,this_host->event_handler_enabled,this_host->flap_detection_enabled,this_host->low_flap_threshold,this_host->high_flap_threshold,this_host->stalk_on_up,this_host->stalk_on_down,this_host->stalk_on_unreachable,this_host->process_perf_data,this_host->failure_prediction_enabled,this_host->failure_prediction_options,this_host->check_freshness,this_host->freshness_threshold,this_host->retain_status_information,this_host->retain_nonstatus_information,this_host->obsess_over_host,
+		this_host->snmp_version,this_host->snmp_login,this_host->snmp_pass,this_host->user_defined);
 
 	/* return with an error if we couldn't add the host */
 	if(new_host==NULL){
@@ -9643,7 +9702,7 @@
 	xodtemplate_hostextinfo *next_hostextinfo;
 	xodtemplate_serviceextinfo *this_serviceextinfo;
 	xodtemplate_serviceextinfo *next_serviceextinfo;
-	int x;
+	int x,i;
 
 #ifdef DEBUG0
 	printf("xodtemplate_free_memory() start\n");
@@ -9777,6 +9836,10 @@
 		free(this_host->contact_groups);
 		free(this_host->notification_period);
 		free(this_host->failure_prediction_options);
+		for (i=0;i<MAX_MACRO_HOSTUSER;i++)
+			free(this_host->user_defined[i]);
+		free(this_host->snmp_login);
+		free(this_host->snmp_pass);
 		free(this_host);
 	        }
 	xodtemplate_host_list=NULL;
--- xdata/xodtemplate.h	Mon Oct 25 07:05:57 2004
+++ xdata/xodtemplate.h	Sun Dec 19 01:47:22 2004
@@ -34,6 +34,7 @@
 #define MAX_XODTEMPLATE_INPUT_BUFFER    65535
 
 #define MAX_XODTEMPLATE_CONTACT_ADDRESSES 6
+#define MAX_MACRO_HOSTUSER			10
 
 #define XODTEMPLATE_NONE                0
 #define XODTEMPLATE_TIMEPERIOD          1
@@ -204,6 +205,10 @@
 	int       have_failure_prediction_enabled;
 	int       have_retain_status_information;
 	int       have_retain_nonstatus_information;
+	int	  snmp_version;
+	char	  *snmp_login;
+	char	  *snmp_pass;
+	char 	  *user_defined[MAX_MACRO_HOSTUSER];
 
 	int       has_been_resolved;
 	int       register_object;
